#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/1/16 16:49
# @Author  : jialun.wjl
# @File    : monitor.py
# @Software: PyCharm
from scapy.all import sniff
from threading import Thread, Event
import urllib3
from logger import logger

class Monitor(Thread):

    def __init__(self, queue, event):
        logger("Initializing network monitor....")
        Thread.__init__(self)
        self.queue = queue
        self.event = event
        self._stop_event = Event()
        self.monitor_url = "unknown"
        self.interface_name = "default"
        self.test_video_url = "None"

    def set_monitor_url(self, url):
        self.monitor_url = url

    def set_report_info(self, url, interface_name, test_video_url):
        self.monitor_url = url
        self.interface_name = interface_name
        self.test_video_url = test_video_url

    def stop(self):
        self._stop_event.set()
        http = urllib3.PoolManager()
        http.request("GET", "http://youku.com")


    def filter_request(self, packet):
        tmp_str = "\n".join(packet.sprintf("{Raw:%Raw.load%}").split(r"\r\n"))
        request_lines = tmp_str.split("\n")
        if len(request_lines) > 1:
            request_uri = request_lines[0]
            origin_host = request_lines[1].split(' ')
            if "gm.mmstat.com" not in origin_host and (("ccode=" in request_uri and "psid=" in request_uri)
                                                       or ("ctype=" in request_uri and "sid=" in request_uri)):
                origin_uri = request_uri.split(' ')
                if len(origin_uri) > 1 and len(origin_host) > 1:
                    self.queue.put([self.monitor_url, "http://" + origin_host[1] + origin_uri[1], self.interface_name,
                                    self.test_video_url])
                    self.event.set()

    def run(self):
        logger("Network sniffer start to run...")
        sniff(prn=self.filter_request, lfilter=lambda p: "GET" in str(p), filter="tcp port 80",
              stop_filter=lambda p: self._stop_event.is_set())
        logger("Network sniffer stopped.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/1/25 20:52
# @Author  : jialun.wjl
# @File    : util.py
# @Software: PyCharm
import time

class Util(object):

    @staticmethod
    def time_now():
        time_array = time.localtime(time.time())
        format_time = time.strftime("%Y%m%d%H%M%S", time_array)
        return format_time



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/4/4 15:54
# @Author  : jialun.wjl
# @File    : site_suite.py
# @Software: PyCharm
from link_analyze.site_module.module_chain import ModuleChain

class SiteSuite(object):

    def __init__(self, webdriver, monitor, video_urls=None):
        site1 = Action5ifxwSite(webdriver, monitor, None, video_urls)
        site2 = SourceSurgeSite(webdriver, monitor, site1, video_urls)
        site3 = Jxwuou0Site(webdriver, monitor, site2, video_urls)
        site4 = FuliWuSite(webdriver, monitor, site3, video_urls)
        self.__chain = site4

    def run(self):
        self.__chain.handle()

class Action5ifxwSite(ModuleChain):

    def __init__(self, webdriver, monitor, successor=None, video_urls=None):
        ModuleChain.__init__(self, "http://www.5ifxw.com/vip/", "doplayers", "url", "/html/body/div[1]/div[" \
                                                                                    "2]/select", webdriver,
                             monitor, 15, successor, video_urls)

    def handle(self):
        ModuleChain.handle()

class SourceSurgeSite(ModuleChain):

    def __init__(self, webdriver, monitor, successor=None, video_urls=None):
        ModuleChain.__init__(self, "http://source.surge.sh/video.html", "bf", "url", '//*[@id="jk"]',
                             webdriver,
                             monitor, 15, successor, video_urls)

    def handle(self):
        ModuleChain.handle()

    def test(self):
        print("test2")

class Jxwuou0Site(ModuleChain):

    def __init__(self, webdriver, monitor, successor=None, video_urls=None):
        ModuleChain.__init__(self, "http://jxw.uou0.com/", "bf", "url", '//*[@id="jk"]', webdriver,
                             monitor, 15, successor, video_urls)

    def handle(self):
        ModuleChain.handle()

class FuliWuSite(ModuleChain):

    def __init__(self, webdriver, monitor, successor=None, video_urls=None):
        ModuleChain.__init__(self, "http://www.fuliw.top/vip/", "doplayers", "url",
                             '/html/body/div[2]/div[2]/select', webdriver,
                             monitor, 15, successor, video_urls)

    def handle(self):
        ModuleChain.handle(self)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/1/24 21:15
# @Author  : jialun.wjl
# @File    : reporter.py
# @Software: PyCharm
from threading import Thread, Event
from util import Util
import openpyxl
import os
import re
from logger import logger

class Reporter(Thread):

    def __init__(self, queue, event):
        print("Initializing reporter....")
        Thread.__init__(self)
        self.queue = queue
        self.event = event
        self.excel = openpyxl.Workbook()
        self._stop_event = Event()
        self._start_time = Util.time_now()
        self._data_array = []
        self._report_dir = "report/"
        self._uniq_list = []

    def stop(self):
        self._stop_event.set()
        self.event.set()

    def run(self):
        while not self._stop_event.is_set():
            while not self.queue.empty():
                data = self.queue.get()
                try:
                    self._parse_one_data(data)
                except Exception as e:
                    print(e)
            self.event.clear()
            self.event.wait()
        if len(self._data_array) != 0:
            self._data_array.insert(0, ['分析源', '请求内容', '线路', '测试视频url', 'ccode', 'psid'])
            self._write_to_excel()

    def _parse_one_data(self, data):
        uk = data[0] + "_" + data[2] + "_" + data[3]
        if uk in self._uniq_list:
            return
        else:
            self._uniq_list.append(uk)
        ccode = ''
        psid = ''
        if "psid=" in data[1]:
            psid = re.findall(r"psid=(.+?)&", data[1])[0]
            ccode = re.findall(r"ccode=(.+?)&", data[1])[0]
        else:
            psid = re.findall(r"sid=(.+?)&", data[1])[0]
            ccode = re.findall(r"ctype=(.+?)&", data[1])[0]
        data.append(ccode)
        data.append(psid)
        self._data_array.append(data)

    def _write_to_excel(self):
        sheet = self.excel.active
        sheet.title = 'madai_link_analyze_report'
        for i in range(0, len(self._data_array)):
            for j in range(0, len(self._data_array[i])):
                sheet.cell(row=i + 1, column=j + 1, value=str(self._data_array[i][j]))
        now = Util.time_now()
        file_name = "分析报告_" + self._start_time + "_" + now + ".xlsx"
        if not os.path.exists(self._report_dir):
            os.mkdir(self._report_dir)
        self.excel.save(self._report_dir + file_name)





#!/usr/bin/env python
# --*--coding: utf-8 --*--

import time
from scapy.all import TCP, IP, sniff, Raw
from logger import logger

class HttpRequestSniffer(object):
    def __init__(self, port=80, iface='en0', filter='tcp', keywords = []):
        self.port = port
        self.iface = iface
        self.filter = filter

        self.http_load = ''
        self.http_fragged = False
        self.http_pack = None
        self.keywords = keywords

    def parser(self, pkt):
        if not pkt.haslayer(Raw):
            return
        elif self.port not in [pkt[TCP].sport, pkt[TCP].dport]:
            return

        self.parse_http(pkt[Raw].load, pkt[IP].ack)

    def parse_http(self, load, ack):
        if ack == self.http_pack:
            self.http_load = self.http_load + load
            load = self.http_load
            self.http_fragged = True
        else:
            self.http_load = load
            self.http_pack = ack
            self.http_fragged = False

        try:
            header_lines = load.split('\r\n\r\n')[0].split('\r\n')
        except ValueError:
            header_lines = load.split('\r\n')

        http_req_url = self.get_http_req_url(header_lines)

        if http_req_url:
            logger(time.strftime('%a, %d %b %Y %H:%M:%S %z: '), http_req_url, True)

    def get_http_req_url(self, header_lines):
        host = ''
        uri = ''
        http_method = header_lines[0][0:header_lines[0].find('/')].strip()

        if http_method != 'GET':
            return

        for line in header_lines:
            # find host
            if 'Host:' in line:
                host = line.split('Host: ')[1].strip()

            # find uri
            if 'GET /' in line:
                uri = line.split('GET ')[1].split(' HTTP/')[0].strip()

        url = ''.join([host, uri])
        if len(self.keywords) == 0:
            return url

        for keyword in self.keywords:
            if keyword in url:
                return url

        return

    def start(self):
        sniff(
            prn=self.parser,
            filter=self.filter,
            iface=self.iface
        )







#!/usr/bin/env python
# -*- coding: utf-8 -*-

from analyzer import Analyzer
from argparse import ArgumentParser
from logger import logger
from httprequestsniffer import HttpRequestSniffer

def main():
    parser = ArgumentParser()
    parser.add_argument('--automodefile', default='', type=str)
    parser.add_argument('--mode', default='auto', type=str)
    parser.add_argument('--keywords', nargs='+')

    args = parser.parse_args()
    logger("param: " + str(args))
    try:
        if args.mode == "auto":
            logger('start Analyzer')
            analyzer = Analyzer()
            analyzer.analyze(args.automodefile)
            logger('finish Analyzer')
        elif args.mode == "manual":
            logger('start HttpRequestSniffer')
            httpRequestSniffer = HttpRequestSniffer(keywords=args.keywords)
            httpRequestSniffer.start()
            logger('finish HttpRequestSniffer')
    except KeyboardInterrupt:
        exit()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/3/7 14:38
# @Author  : jialun.wjl
# @File    : module_chain.py
# @Software: PyCharm
from time import sleep
from selenium.webdriver.support.select import Select


class ModuleChain(object):

    def __init__(self, web_url, submit_btn_id, url_input_id, select_xpath, webdriver, monitor, sleep_time=15,
                 successor=None, video_urls=None):
        self.__web_url = web_url
        self.__submit_id = submit_btn_id
        self.__url_input_id = url_input_id
        self.__select_xpath = select_xpath
        self.__monitor = monitor
        self.__webdriver = webdriver
        self.__sleep_time = sleep_time
        self.__successor = successor
        self.__video_urls = video_urls

    def set_successor(self, successor):
        self.__successor = successor

    def get_driver(self):
        return self.__webdriver

    def get_video_urls(self):
        return self.__video_urls

    def get_monitor(self):
        return self.__monitor

    def handle(self):
        # todo
        print("Analyzing web site : ", self.__web_url)
        self.__webdriver.get(self.__web_url)
        url_input = self.__webdriver.find_element_by_id(self.__url_input_id)
        submit_btn = self.__webdriver.find_element_by_id(self.__submit_id)
        channel_select = Select(self.__webdriver.find_element_by_xpath(self.__select_xpath))
        select_num = len(channel_select.options)
        for i in range(select_num):
            channel_select.select_by_index(i)
            for url in self.__video_urls:
                try:
                    self.__monitor.set_report_info(self.__web_url, channel_select.first_selected_option.text, url)
                    url_input.clear()
                    url_input.send_keys(url)
                    submit_btn.click()
                    sleep(self.__sleep_time)
                except Exception as e:
                    print(e)
        if self.__successor is not None:
            self.__successor.handle()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2018/1/16 17:30
# @Author  : jialun.wjl
# @File    : analyzer.py
# @Software: PyCharm
from selenium import webdriver
from monitor import Monitor
from reporter import Reporter
from threading import Event
from Queue import Queue
import json
import time
from logger import logger
from selenium.webdriver.firefox.firefox_binary import FirefoxBinary

class Analyzer(object):

    def __init__(self):
        self.queue = Queue()
        self.event = Event()
        self.monitor = Monitor(self.queue, self.event)
        self.reporter = Reporter(self.queue, self.event)

    def analyze(self, file_path):
        auto_mode_str = json.loads(open(file_path, "r").read())

        logger("auto_mode_str" + str(auto_mode_str))
        self.profile = webdriver.FirefoxProfile(auto_mode_str["firefoxConfig"])


        self.profile.set_preference('dom.ipc.plugins.enabled.libflashplayer.so', 'true')
        self.profile.set_preference("plugin.state.flash", 2)
        self.driver = webdriver.Firefox(self.profile,executable_path='./browser/mac/geckodriver')
        self.driver.implicitly_wait(30)

        self.monitor.start()
        self.reporter.start()

        for url in auto_mode_str["urls"]:
            logger("Analyzing url : ", url)
            try:
                self.driver.get(url)
                self.monitor.set_monitor_url(url)
                time.sleep(20)
            except Exception as e:
                logger(e)
        # additional_sites = site_suite.SiteSuite(self.driver, self.monitor,
        #                                         ["http://v.youku.com/v_show/id_XMzQ1MjM5ODY0NA==.html"])
        #
        # additional_sites.run()
        self.monitor.stop()
        self.reporter.stop()
        self.driver.quit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : pengjun.pj

from setuptools import setup, find_packages

setup(
    name='madai-link-analyze',
    version='0.0.0.2',
    description='for madai link theft analyze, input url , output theft url',
    author='pengjun.pj',
    author_email='pengjun.pj@alibaba-inc.com',
    url='',
    py_modules=['madai-link-analyze'],
    install_requires=[
        "openpyxl == 2.4.9", "selenium==3.7.0","scapy==2.3.2","urllib3==1.22"
    ],
    python_requires=">=2.6, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    license='GPL',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    entry_points={'console_scripts': [
        'entry = link_analyze.main:main']},
)



# --*--coding: utf-8 --*--

import os

REQ_URLS_FILE = open('madai-link-analyze.txt', 'a')

def logger(title, message='', write=False):
    w = '\033[0m'
    g = '\033[32m'
    # write log message to log file if `write == true`
    if write:
        REQ_URLS_FILE.write(''.join([message, '\n']))
        REQ_URLS_FILE.flush()
        os.fsync(REQ_URLS_FILE.fileno())
    print(''.join([g, title, w, message]))


